﻿using ProjectDumpTruck.CollectionGenericObjects;
using ProjectDumpTruck.Drawnings;

namespace ProjectDumpTruck;

/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormTruckCollection : Form
{
    /// <summary>
	/// Хранилище коллекций
	/// </summary>
	private readonly StorageCollection<DrawningTruck> _storageCollection;


    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTruckCollection()
    {
        InitializeComponent();
        _storageCollection = new();
    }

    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }

    /// <summary>
    /// Создание объекта класса-перемещения
    /// </summary>
    /// <param name="type">Тип создаваемого объекта</param>
    private void CreateObject(string type)
    {
        if (_company == null)
        {
            return;
        }

        Random random = new();
        DrawningTruck drawningTruck;
        switch (type)
        {
            case nameof(DrawningTruck):
                drawningTruck = new DrawningTruck(random.Next(100, 300), random.Next(1000, 3000), GetColor(random));
                break;
            case nameof(DrawningDumpTruck):
                // TODO Выбор цвета
                drawningTruck = new DrawningDumpTruck(random.Next(100, 300), random.Next(1000, 3000), GetColor(random), GetColor(random), GetColor(random),
                    Convert.ToBoolean(random.Next(2, 2)), Convert.ToBoolean(random.Next(1, 2)));
                break;
            default:
                return;
        }
        if (_company + drawningTruck != -1)
        {
            MessageBox.Show("Объект добавлен");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось добавить объект");
        }
    }

    /// <summary>
    /// Получение цвета
    /// </summary>
    /// <param name="random">Генератор случайных чисел</param>
    /// <returns></returns>
    private static Color GetColor(Random random)
    {
        Color color = Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256));
        ColorDialog dialog = new();
        if (dialog.ShowDialog() == DialogResult.OK)
        {
            color = dialog.Color;
        }

        return color;
    }

    /// <summary>
    ///  Добавление грузовика
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddTruck_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningTruck));

    /// <summary>
    /// Добавление самосвала
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddDumpTruck_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningDumpTruck));

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveTruck_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }

        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }

        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        if (_company - pos != null)
        {
            MessageBox.Show("Объект удален");
            pictureBox.Image = _company.Show();
        }
        else
        {
            MessageBox.Show("Не удалось удалить объект");
        }
    }

    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        DrawningTruck? truck = null;
        int counter = 100;
        while (truck == null)
        {
            truck = _company.GetRandomObject();
            counter--;
            if (counter <= 0)
            {
                break;
            }
        }

        if (truck == null)
        {
            return;
        }

        FormTransport form = new FormTransport();
        form.SetTruck = truck;
        form.ShowDialog();
    }

    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }

        pictureBox.Image = _company.Show();
    }

    /// <summary>
	/// Добавление коллекции
	/// </summary>
	/// <param name="sender"></param>
	/// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }

        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }

        _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
        RerfreshListBoxItems();

    }

    /// <summary>
    /// Обновление списка в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }

    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        // TODO прописать логику удаления элемента из коллекции
        // нужно убедиться, что есть выбранная коллекция
        // спросить у пользователя через MessageBox, что он подтверждает, что хочет удалить запись
        // удалить и обновить ListBox

        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        RerfreshListBoxItems();

    }

    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        ICollectionGenericObjects<DrawningTruck>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }

        switch (comboBoxSelectorCompany.Text)
        {
            case "Хранилище":
                _company = new TruckParkingService(pictureBox.Width, pictureBox.Height, collection);
                break;
        }

        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }
}