﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectDumpTruck.Drawnings;

namespace ProjectDumpTruck.MovementStrategy;

/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawningTruck
/// </summary>
public class MoveableTruck : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawningShip или его наследника
    /// </summary>
    private readonly DrawningTruck? _truck = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="truck"></param>
    public MoveableTruck(DrawningTruck truck)
    {
        _truck = truck;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_truck == null || _truck.EntityTruck == null || !_truck.GetPosX.HasValue || !_truck.GetPosY.HasValue)
            {
                return null;
            }

            return new ObjectParameters(_truck.GetPosX.Value, _truck.GetPosY.Value, _truck.GetWidth, _truck.GetHeight);
        }
    }

    public int GetStep => (int)(_truck?.EntityTruck?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_truck == null || _truck.EntityTruck == null)
        {
            return false;
        }

        return _truck.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
